/*
	Pinguino + MOD-GPS 
	Example
	
	If you have any questions, email
	support@olimex.com
	
	|------------------------------------------
	| v. 1.0 - initial version
	| v. 1.1 - Fixed recording/printing issues
	|------------------------------------------
	
	Olimex, June 2012
	http://www.olimex.com/dev
	
*/

#include <serial.c>
#include <__cdc.c>

#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) || defined(PIC32_PINGUINO_MICRO) )

	//Definitions for 
	// -- PIC32-PINGUINO
	// -- PIC32-PINGUINO-OTG
	// -- PIC32-PINGUINO-MICRO

	
	#define gps_port UART2      //The UEXT is on UART2
	#define gps_baud 19200
	#define BUTTON1INIT TRISDbits.TRISD0 = 1
	
	//YELLOW LED is D1
	#define YLEDINIT   TRISDCLR = 0x02;
	#define YLED1      PORTDSET = 0x02;
	#define YLED0      PORTDCLR = 0x02;
	#define YLEDSWITCH PORTD   ^= 0x02;
	
	//GREEN LED is B15
	#define GLEDINIT  	TRISGCLR = 0x40;
	#define GLED1 		PORTGSET = 0x40;
	#define GLED0 		PORTGCLR = 0x40;
	#define GLEDSWITCH  PORTG   ^= 0x40;
	
	#define YELLOW 0
	#define GREEN  1
	#define BOTH   2
#endif

/*  Prototypes and useful functions     */


	//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
	void blinkled(unsigned char mode);

	//Catches a string and writes to the buffer
	//The module sends lines in the format <string><CR><LF>
	void GPS_get_response(unsigned char uport);
 
    //Prints text from a given index in the buffer
	//until it meets the ',' (comma) character
	//returns the new index
	unsigned char print_till_comma(const char *buf, volatile unsigned char a);
	
	//Prints the data received in the buffer
	//in a user-frinedly style
	void print_data(char *buf);
	

	//A macro that waits for a key via CDC
	//Works with all pinguinos that support CDC
	//Syntax:
	//   WAIT;   //-> pauses the program and waits
				 //   until a key is pressed 
		
	//WAIT Macro 
	#define WAIT wait_123()
	char ch123;
	
	void wait_123() __attribute ((noinline));
	
	void wait_123(){ ch123 = CDCgetkey(); }
	//END Wait Macro
	
	// Sends a single line of text to the terminal via CDC
	void message(char * m){ CDCprintln(m);}
	
	
/*		Global variables			  */

	#define SERIAL_BUFFER_SIZE 80
	volatile unsigned char serial_buffer[SERIAL_BUFFER_SIZE];
	
/*	    Interrupt routine definition  */

	// Interrupt routine checks if a string has been received
	// from the module and prints it on the terminal
	// Connect the wrapper to vector 4 (Timer1 overflow)
	void ISR_wrapper_vector_4(void) __attribute__ ((section (".vector_4")));

	// The wrapper calls the "Tmr1Interrupt" function
	// each time an interrupt occurs
	void ISR_wrapper_vector_4(void) { Tmr1Interrupt(); }

	//We declare the function as an interrupt routine
	void Tmr1Interrupt(void) __attribute__ ((interrupt));
	
	// configure timer 1
	void init_timer1(void)
	{
		IntConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);	// interrupt mode (interrupt.c)
		T1CON=0;			// reset timer 1 configuration
		TMR1=0;				// reset timer 1 counter register
		PR1=0x9896;			// define the period (an interrupt will occur ~ 1024 times a second)
		IPC1SET=0x7;		// select interrupt priority and sub-priority
		IFS0CLR=0x10;		// clear interrupt flag
		IEC0SET=0x10;		// enable timer 1 interrupt
		T1CONCLR=0x30;	  	// set prescaler to 1:1
		T1CONSET=0x8000; 	// turn TIMER1 ON
	}
	
 volatile unsigned char counter=0;	
	// The interrupt routine code			
	void Tmr1Interrupt(void)
	{
		T1CONCLR = (1<<15);  //Stop Timer 1
		
		//If a string has been received, print it on the terminal
		if(SerialAvailable(gps_port))
		{
			GPS_get_response(gps_port);
     
			//serial_buffer now has the string
			if( (serial_buffer[4] == 'G') && (serial_buffer[5] == 'A') )
			   {
				  counter=0;
				  print_data(serial_buffer); //if it's the GPGLL string, print the data
           				 
			    }else
				 counter++;
           
			if(counter == 48)
			{
				counter=0;
				message("Signal NA");
			}
			
		}
		//turn Timer 1 on again
		init_timer1();
	}
	
	

void setup()
{	
	WAIT;
	message("Press a key to init the GPS module");
	WAIT;
	
	/* Init the Module, LEDs and Button on the pinguino board*/
		YLEDINIT;
		GLEDINIT;	
		YLED0;
		GLED0;
		BUTTON1INIT;
		
	message("Press enter to start printing...");
	
	// Open a serial communication from the board to the module
		SerialConfigure(gps_port, UART_ENABLE, UART_RX_TX_ENABLED, gps_baud);
	
	init_timer1();
}

void loop(){}

void blinkled(unsigned char mode)
{		
	//Turn off all LEDs
	YLED0;
	GLED0;
	
	char i;
	
	// Blink
	switch(mode)
	{
		case YELLOW:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				delay(100);
			}
			break;
		
		case GREEN:
			for(i=0; i<10;i++)
			{
				GLEDSWITCH;
				delay(100);
			}
			break;
		
		case BOTH:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				GLEDSWITCH;
				delay(100);
			}
			break;
	}	
					
				
}

void GPS_get_response(unsigned char uport)
{
		char i = 0;
		char c;
		
		do{
			c = SerialGetKey(uport);
		  }while(c!='$'); // Only start recording at the beginning of the string
		  
		  serial_buffer[i] = c; i++;
			
	// The rest of the code is based on the 
	// SerialGetString function in the 
	// <serial.c> library
		do {
				c = SerialGetKey(uport);
				
			    // Stop when you meet the <CR> character
				if( c != '\r' && c != '$' )
				{	
					serial_buffer[i] = c;
					i++;
				}
				
			}while( c != '\r' && c != '$');
		serial_buffer[i] = '\0';
		
	 //Catch the <LF> character
		c = SerialGetKey(uport);		
}

unsigned char print_till_comma(const char *buf, volatile unsigned char a)
{
	if(buf[a] == ',')
	{
		CDC.println("NA");
		return a+1; 
	}
	else
	{
		do{
			CDC.printf("%c", buf[a]);
			a++;
		  }while( buf[a] != ',' && buf[a] != '\0');
    message("");
		  
		return a+1;
	}
		
}

void print_data(char *buf)
{
	volatile unsigned char index = 7;
	
		//DEBUG:
		message(buf);
	
	  message("");
	  message("*--------------------------------------*");
	  message("Printing GPGGA geo data from the module: ");
	CDC.print("            UTC Time: "); if(buf[index] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("            Latitude: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("            N/S ind.: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("           Longitude: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("             E/W ind: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("    Position fix ind: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("     Satellites Used: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("                HDOP: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("        MSL Altitude: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("               Units: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("    Geoid Separation: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("               Units: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("   Age of Diff. Corr: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("Diff. reference Corr: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	CDC.print("            Checksum: "); if(buf[index-1] != '\0') index = print_till_comma(buf, index); else message("NA");
	message("*--------------------------------------*");
	message("");
	
	char i;
	for(i=0; i<SERIAL_BUFFER_SIZE; i++)  //make sure there are no remaining numbers in the buffer
		buf[i] = ',';
}
